@@ -1,10 +1,11 @@
 #include "recv_append_entries_result.h"
 #include "assert.h"
 #include "configuration.h"
-#include "tracing.h"
 #include "recv.h"
 #include "replication.h"
+#include "tracing.h"
 
+#define infof(...) Infof(r->tracer, "  " __VA_ARGS__)
 #define tracef(...) Tracef(r->tracer, __VA_ARGS__)
 
 int recvAppendEntriesResult(struct raft *r,
@@ -12,8 +13,8 @@ int recvAppendEntriesResult(struct raft *r,
                             const char *address,
                             const struct raft_append_entries_result *result)
 {
-    int match;
     const struct raft_server *server;
+    int match;
     int rv;
 
     assert(r != NULL);
@@ -21,21 +22,26 @@ int recvAppendEntriesResult(struct raft *r,
     assert(address != NULL);
     assert(result != NULL);
 
-    tracef("self:%llu from:%llu@%s last_log_index:%llu rejected:%llu term:%llu",
-            r->id, id, address, result->last_log_index, result->rejected, result->term);
+    /* XXX: Up to version 0.19.1 followers were erroneously setting
+     * last_log_index to whatever their last log index was, regardless or the
+     * index being rejected. If we detect such a case, we manually amend it
+     * here. This code can be dropped once sufficient time has passed that we
+     * are confident that no server is running the old buggy code. */
+    if (result->rejected > 0 && result->last_log_index >= result->rejected) {
+        ((struct raft_append_entries_result *)result)->last_log_index =
+            result->rejected - 1;
+    }
 
     if (r->state != RAFT_LEADER) {
-        tracef("local server is not leader -> ignore");
+        infof("local server is not leader -> ignore");
         return 0;
     }
 
-    rv = recvEnsureMatchingTerms(r, result->term, &match);
-    if (rv != 0) {
-        return rv;
-    }
+    match = recvEnsureMatchingTerms(r, result->term);
 
     if (match < 0) {
-        tracef("local term is higher -> ignore ");
+        infof("local term is higher (%llu vs %llu) -> ignore", r->current_term,
+              result->term);
         return 0;
     }
 
@@ -56,7 +62,7 @@ int recvAppendEntriesResult(struct raft *r,
     /* Ignore responses from servers that have been removed */
     server = configurationGet(&r->configuration, id);
     if (server == NULL) {
-        tracef("unknown server -> ignore");
+        infof("unknown server -> ignore");
         return 0;
     }
 
@@ -69,4 +75,5 @@ int recvAppendEntriesResult(struct raft *r,
     return 0;
 }
 
+#undef infof
 #undef tracef
