@@ -2,25 +2,19 @@
 
 #include "assert.h"
 #include "convert.h"
-#include "log.h"
+#include "message.h"
 #include "recv.h"
 #include "replication.h"
 #include "tracing.h"
+#include "trail.h"
 
-#define tracef(...) Tracef(r->tracer, __VA_ARGS__)
-
-static void installSnapshotSendCb(struct raft_io_send *req, int status)
-{
-    (void)status;
-    raft_free(req);
-}
+#define infof(...) Infof(r->tracer, "  " __VA_ARGS__)
 
 int recvInstallSnapshot(struct raft *r,
                         const raft_id id,
                         const char *address,
                         struct raft_install_snapshot *args)
 {
-    struct raft_io_send *req;
     struct raft_message message;
     struct raft_append_entries_result *result = &message.append_entries_result;
     int rv;
@@ -28,19 +22,15 @@ int recvInstallSnapshot(struct raft *r,
     bool async;
 
     assert(address != NULL);
-    tracef("self:%llu from:%llu@%s conf_index:%llu last_index:%llu last_term:%llu term:%llu",
-            r->id, id, address, args->conf_index, args->last_index, args->last_term, args->term);
 
-    result->rejected = args->last_index;
-    result->last_log_index = logLastIndex(&r->log);
+    result->version = MESSAGE__APPEND_ENTRIES_RESULT_VERSION;
+    result->features = MESSAGE__FEATURE_CAPACITY;
 
-    rv = recvEnsureMatchingTerms(r, args->term, &match);
-    if (rv != 0) {
-        return rv;
-    }
+    match = recvEnsureMatchingTerms(r, args->term);
 
     if (match < 0) {
-        tracef("local term is higher -> reject ");
+        infof("local term is higher (%llu vs %llu) -> reject", r->current_term,
+              args->term);
         goto reply;
     }
 
@@ -49,7 +39,7 @@ int recvInstallSnapshot(struct raft *r,
     assert(r->current_term == args->term);
     if (r->state == RAFT_CANDIDATE) {
         assert(match == 0);
-        tracef("discovered leader -> step down ");
+        infof("discovered leader (%llu) -> step down ", id);
         convertToFollower(r);
     }
 
@@ -57,11 +47,11 @@ int recvInstallSnapshot(struct raft *r,
     if (rv != 0) {
         return rv;
     }
-    r->election_timer_start = r->io->time(r->io);
+    r->election_timer_start = r->now;
+    r->update->flags |= RAFT_UPDATE_TIMEOUT;
 
-    rv = replicationInstallSnapshot(r, args, &result->rejected, &async);
+    rv = replicationInstallSnapshot(r, args, &async);
     if (rv != 0) {
-        tracef("replicationInstallSnapshot failed %d", rv);
         return rv;
     }
 
@@ -69,35 +59,37 @@ int recvInstallSnapshot(struct raft *r,
         return 0;
     }
 
-    if (result->rejected == 0) {
-        /* Echo back to the leader the point that we reached. */
-        result->last_log_index = args->last_index;
+    /* If we got here it mess that we either have a more recent snapshot than
+     * the one being sent, or that we already have all snapshot entries in our
+     * log. */
+    assert(TrailLastIndex(&r->trail) >= args->last_index);
+
+    /* Echo back to the leader the point that we reached. */
+    result->last_log_index = args->last_index;
+    if (r->last_stored < result->last_log_index) {
+        result->last_log_index = r->last_stored;
     }
 
 reply:
     result->term = r->current_term;
+    result->rejected = 0;
 
     /* Free the snapshot data. */
     raft_configuration_close(&args->conf);
     raft_free(args->data.base);
 
-    message.type = RAFT_IO_APPEND_ENTRIES_RESULT;
+    result->capacity = r->capacity;
+
+    message.type = RAFT_APPEND_ENTRIES_RESULT;
     message.server_id = id;
     message.server_address = address;
 
-    req = raft_malloc(sizeof *req);
-    if (req == NULL) {
-        return RAFT_NOMEM;
-    }
-    req->data = r;
-
-    rv = r->io->send(r->io, req, &message, installSnapshotSendCb);
+    rv = MessageEnqueue(r, &message);
     if (rv != 0) {
-        raft_free(req);
         return rv;
     }
 
     return 0;
 }
 
-#undef tracef
+#undef infof
