@@ -3,11 +3,11 @@
 #include "assert.h"
 #include "configuration.h"
 #include "convert.h"
-#include "log.h"
 #include "recv.h"
 #include "tracing.h"
+#include "trail.h"
 
-#define tracef(...) Tracef(r->tracer, __VA_ARGS__)
+#define infof(...) Infof(r->tracer, "  " __VA_ARGS__)
 
 int recvTimeoutNow(struct raft *r,
                    const raft_id id,
@@ -26,11 +26,10 @@ int recvTimeoutNow(struct raft *r,
 
     (void)address;
 
-    tracef("self:%llu from:%llu@%s last_log_index:%llu last_log_term:%llu term:%llu", r->id, id, address, args->last_log_index, args->last_log_term, args->term);
     /* Ignore the request if we are not voters. */
     local_server = configurationGet(&r->configuration, r->id);
     if (local_server == NULL || local_server->role != RAFT_VOTER) {
-        tracef("non-voter");
+        infof("non-voter");
         return 0;
     }
 
@@ -38,29 +37,35 @@ int recvTimeoutNow(struct raft *r,
      * leader. */
     if (r->state != RAFT_FOLLOWER ||
         r->follower_state.current_leader.id != id) {
-        tracef("Ignore - r->state:%d current_leader.id:%llu", r->state, r->follower_state.current_leader.id);
+        infof("ignore - r->state:%d current_leader.id:%llu", r->state,
+              r->follower_state.current_leader.id);
         return 0;
     }
 
     /* Possibly update our term. Ignore the request if it turns out we have a
      * higher term. */
-    rv = recvEnsureMatchingTerms(r, args->term, &match);
-    if (rv != 0) {
-        return rv;
-    }
+    match = recvEnsureMatchingTerms(r, args->term);
     if (match < 0) {
         return 0;
     }
 
     /* Ignore the request if we our log is not up-to-date. */
-    local_last_index = logLastIndex(&r->log);
-    local_last_term = logLastTerm(&r->log);
+    local_last_index = TrailLastIndex(&r->trail);
+    local_last_term = TrailLastTerm(&r->trail);
     if (local_last_index != args->last_log_index ||
         local_last_term != args->last_log_term) {
         return 0;
     }
 
+    /* Ignore the request if we're still persisting entries or installing a
+     * snapshot. */
+    if (r->last_stored < local_last_index || r->snapshot.installing) {
+        return 0;
+    }
+
     /* Convert to candidate and start a new election. */
+    infof("convert to candidate, start election for term %llu",
+          r->current_term + 1);
     rv = convertToCandidate(r, true /* disrupt leader */);
     if (rv != 0) {
         return rv;
@@ -69,4 +74,4 @@ int recvTimeoutNow(struct raft *r,
     return 0;
 }
 
-#undef tracef
+#undef infof
