@@ -11,6 +11,10 @@ struct request
     int type;
     raft_index index;
     void *queue[2];
+    uint8_t req_id[16];
+    uint8_t client_id[16];
+    uint8_t unique_id[16];
+    uint64_t reserved[4];
 };
 
 #endif /* REQUEST_H_ */
