@@ -1,6 +1,5 @@
 #include <stdlib.h>
 #include <string.h>
-#include <sys/uio.h>
 
 #include "array.h"
 #include "assert.h"
@@ -32,11 +31,11 @@ static bool uvSnapshotParseFilename(const char *filename,
     size_t filename_len = strlen(filename);
     assert(filename_len < UV__FILENAME_LEN);
     if (meta) {
-        matched = sscanf(filename, UV__SNAPSHOT_META_TEMPLATE "%n",
-                         term, index, timestamp, &consumed);
+        matched = sscanf(filename, UV__SNAPSHOT_META_TEMPLATE "%n", term, index,
+                         timestamp, &consumed);
     } else {
-        matched = sscanf(filename, UV__SNAPSHOT_TEMPLATE "%n",
-                         term, index, timestamp, &consumed);
+        matched = sscanf(filename, UV__SNAPSHOT_TEMPLATE "%n", term, index,
+                         timestamp, &consumed);
     }
     if (matched != 3 || consumed != (int)filename_len) {
         return false;
@@ -53,7 +52,8 @@ static bool uvSnapshotParseFilename(const char *filename,
 static bool uvSnapshotInfoMatch(const char *filename,
                                 struct uvSnapshotInfo *info)
 {
-    if (!uvSnapshotParseFilename(filename, true, &info->term, &info->index, &info->timestamp)) {
+    if (!uvSnapshotParseFilename(filename, true, &info->term, &info->index,
+                                 &info->timestamp)) {
         return false;
     }
     /* Allow room for '\0' terminator */
@@ -132,26 +132,34 @@ int UvSnapshotInfoAppendIfMatch(struct uv *uv,
     return 0;
 }
 
-static int uvSnapshotIsOrphanInternal(const char *dir, const char *filename, bool meta, bool *orphan)
+static int uvSnapshotIsOrphanInternal(const char *dir,
+                                      const char *filename,
+                                      bool meta,
+                                      bool *orphan)
 {
-    int rv;
-    *orphan = false;
-
     raft_term term;
     raft_index index;
     raft_time timestamp;
-    if (!uvSnapshotParseFilename(filename, meta, &term, &index, &timestamp)) {
+    bool is_snapshot_filename;
+    int rv;
+
+    *orphan = false;
+
+    is_snapshot_filename =
+        uvSnapshotParseFilename(filename, meta, &term, &index, &timestamp);
+    if (!is_snapshot_filename) {
         return 0;
     }
 
-    /* filename is a well-formed snapshot filename, check if the sibling file exists. */
+    /* filename is a well-formed snapshot filename, check if the sibling file
+     * exists. */
     char sibling_filename[UV__FILENAME_LEN];
     if (meta) {
         rv = snprintf(sibling_filename, UV__FILENAME_LEN, UV__SNAPSHOT_TEMPLATE,
                       term, index, timestamp);
     } else {
-        rv = snprintf(sibling_filename, UV__FILENAME_LEN, UV__SNAPSHOT_META_TEMPLATE,
-                      term, index, timestamp);
+        rv = snprintf(sibling_filename, UV__FILENAME_LEN,
+                      UV__SNAPSHOT_META_TEMPLATE, term, index, timestamp);
     }
 
     if (rv >= UV__FILENAME_LEN) {
@@ -288,7 +296,6 @@ static int uvSnapshotLoadMeta(struct uv *uv,
         goto err_after_buf_malloc;
     }
 
-    configurationInit(&snapshot->configuration);
     rv = configurationDecode(&buf, &snapshot->configuration);
     if (rv != 0) {
         goto err_after_buf_malloc;
@@ -382,15 +389,17 @@ struct uvSnapshotPut
     size_t trailing;
     struct raft_io_snapshot_put *req;
     const struct raft_snapshot *snapshot;
+    uv_file snapshot_fd; /* Pre-allocated snapshot file */
     struct
     {
         unsigned long long timestamp;
         uint64_t header[4];         /* Format, CRC, configuration index/len */
         struct raft_buffer bufs[2]; /* Preamble and configuration */
+        uv_file fd;                 /* Pre-allocated metadata temp file */
     } meta;
     char errmsg[RAFT_ERRMSG_BUF_SIZE];
     int status;
-    struct UvBarrier barrier;
+    struct UvBarrierReq barrier;
 };
 
 struct uvSnapshotGet
@@ -476,26 +485,6 @@ out:
     return rv;
 }
 
-static int makeFileCompressed(const char *dir,
-                           const char *filename,
-                           struct raft_buffer *bufs,
-                           unsigned n_bufs,
-                           char *errmsg)
-{
-    int rv;
-
-    struct raft_buffer compressed = {0};
-    rv = Compress(bufs, n_bufs, &compressed, errmsg);
-    if (rv != 0) {
-        ErrMsgWrapf(errmsg, "compress %s", filename);
-        return RAFT_IOERR;
-    }
-
-    rv = UvFsMakeFile(dir, filename, &compressed, 1, errmsg);
-    raft_free(compressed.base);
-    return rv;
-}
-
 static void uvSnapshotPutWorkCb(uv_work_t *work)
 {
     struct uvSnapshotPut *put = work->data;
@@ -508,10 +497,10 @@ static void uvSnapshotPutWorkCb(uv_work_t *work)
     sprintf(metadata, UV__SNAPSHOT_META_TEMPLATE, put->snapshot->term,
             put->snapshot->index, put->meta.timestamp);
 
-    rv = UvFsMakeFile(uv->dir, metadata, put->meta.bufs, 2, put->errmsg);
+    rv = UvFsFinalizeTempFile(put->meta.fd, uv->dir, metadata, put->errmsg);
     if (rv != 0) {
-        tracef("snapshot.meta creation failed %d", rv);
-        ErrMsgWrapf(put->errmsg, "write %s", metadata);
+        ErrMsgWrapf(put->errmsg, "finalize %s", metadata);
+        tracef("snapshot.meta creation failed: %s", put->errmsg);
         put->status = RAFT_IOERR;
         return;
     }
@@ -519,19 +508,11 @@ static void uvSnapshotPutWorkCb(uv_work_t *work)
     sprintf(snapshot, UV__SNAPSHOT_TEMPLATE, put->snapshot->term,
             put->snapshot->index, put->meta.timestamp);
 
-    tracef("snapshot write start");
-    if (uv->snapshot_compression) {
-        rv = makeFileCompressed(uv->dir, snapshot, put->snapshot->bufs,
-                                put->snapshot->n_bufs, put->errmsg);
-    } else {
-        rv = UvFsMakeFile(uv->dir, snapshot, put->snapshot->bufs,
-                          put->snapshot->n_bufs, put->errmsg);
-    }
+    rv = UvFsFinalizeTempFile(put->snapshot_fd, uv->dir, snapshot, put->errmsg);
     tracef("snapshot write end %d", rv);
-
     if (rv != 0) {
         tracef("snapshot creation failed %d", rv);
-        ErrMsgWrapf(put->errmsg, "write %s", snapshot);
+        ErrMsgWrapf(put->errmsg, "finalize %s", snapshot);
         UvFsRemoveFile(uv->dir, metadata, errmsg);
         UvFsRemoveFile(uv->dir, snapshot, errmsg);
         put->status = RAFT_IOERR;
@@ -600,15 +581,16 @@ static void uvSnapshotPutStart(struct uvSnapshotPut *put)
     }
 }
 
-static void uvSnapshotPutBarrierCb(struct UvBarrier *barrier)
+static void uvSnapshotPutBarrierCb(struct UvBarrierReq *barrier)
 {
+    /* Ensure that we don't invoke this callback more than once. */
+    barrier->cb = NULL;
     struct uvSnapshotPut *put = barrier->data;
     if (put == NULL) {
         return;
     }
 
     struct uv *uv = put->uv;
-    assert((barrier->blocking && put->trailing == 0) || !barrier->blocking);
     put->barrier.data = NULL;
     /* If we're closing, abort the request. */
     if (uv->closing) {
@@ -620,6 +602,100 @@ static void uvSnapshotPutBarrierCb(struct UvBarrier *barrier)
     uvSnapshotPutStart(put);
 }
 
+static void uvSnapshotPutWorkAllocateCb(uv_work_t *work)
+{
+    struct uvSnapshotPut *put = work->data;
+    struct uv *uv = put->uv;
+    const struct raft_snapshot *snapshot = put->snapshot;
+    int rv;
+
+    rv = UvFsCreateTempFile(uv->dir, put->meta.bufs, 2, &put->meta.fd,
+                            put->errmsg);
+    if (rv != 0) {
+        goto abort;
+    }
+
+    rv = UvFsCreateTempFile(uv->dir, snapshot->bufs, snapshot->n_bufs,
+                            &put->snapshot_fd, put->errmsg);
+    if (rv != 0) {
+        goto abort_after_meta_open;
+    }
+
+    put->status = 0;
+
+    return;
+
+abort_after_meta_open:
+    UvOsClose(put->meta.fd);
+abort:
+    put->status = rv;
+}
+
+static void uvSnapshotPutAfterWorkAllocateCb(uv_work_t *work, int status);
+
+static void uvSnapshotPutRetryTimerCb(uv_timer_t *timer)
+{
+    struct uvSnapshotPut *put = timer->data;
+    struct uv *uv = put->uv;
+    int rv;
+    uv->snapshot_put_retry.data = uv;
+    uv->snapshot_put_work.data = put;
+    rv = uv_queue_work(uv->loop, &uv->snapshot_put_work,
+                       uvSnapshotPutWorkAllocateCb,
+                       uvSnapshotPutAfterWorkAllocateCb);
+    assert(rv == 0);
+}
+
+static void uvSnapshotPutAfterWorkAllocateCb(uv_work_t *work, int status)
+{
+    struct uvSnapshotPut *put = work->data;
+    const struct raft_snapshot *snapshot = put->snapshot;
+    struct uv *uv = put->uv;
+    raft_index next_index;
+    int rv;
+
+    assert(status == 0);
+    uv->snapshot_put_work.data = NULL;
+
+    if (uv->closing) {
+        put->status = RAFT_CANCELED;
+        goto abort;
+    }
+
+    if (put->status != 0) {
+        assert(uv->snapshot_put_retry.data == uv);
+        uv->snapshot_put_retry.data = put;
+        tracef("retry snapshot write");
+        rv = uv_timer_start(&uv->snapshot_put_retry, uvSnapshotPutRetryTimerCb,
+                            uv->disk_retry, 0);
+        assert(rv == 0);
+        return;
+    }
+
+    /* - If the trailing parameter is set to 0, it means that we're restoring a
+     * snapshot. Submit a barrier request setting the next append index to the
+     * snapshot's last index + 1.
+     * - When we are only writing a snapshot during normal operation, we close
+     * all current open segments. New writes can continue on newly opened
+     * segments that will only contain entries that are newer than the snapshot,
+     * and we don't change append_next_index. */
+    next_index =
+        (put->trailing == 0) ? (snapshot->index + 1) : uv->append_next_index;
+
+    rv = UvBarrier(uv, next_index, &put->barrier);
+    if (rv != 0) {
+        put->status = rv;
+        goto abort;
+    }
+
+    return;
+
+abort:
+    assert(put->status != 0);
+    uvSnapshotPutFinish(put);
+    UvUnblock(uv);
+}
+
 int UvSnapshotPut(struct raft_io *io,
                   unsigned trailing,
                   struct raft_io_snapshot_put *req,
@@ -628,10 +704,9 @@ int UvSnapshotPut(struct raft_io *io,
 {
     struct uv *uv;
     struct uvSnapshotPut *put;
-    void *cursor;
+    uint8_t *cursor;
     unsigned crc;
     int rv;
-    raft_index next_index;
 
     uv = io->impl;
     if (uv->closing) {
@@ -654,6 +729,7 @@ int UvSnapshotPut(struct raft_io *io,
     put->trailing = trailing;
     put->barrier.data = put;
     put->barrier.blocking = trailing == 0;
+    put->barrier.cb = uvSnapshotPutBarrierCb;
 
     req->cb = cb;
 
@@ -666,7 +742,7 @@ int UvSnapshotPut(struct raft_io *io,
         goto err_after_req_alloc;
     }
 
-    cursor = put->meta.header;
+    cursor = (uint8_t *)put->meta.header;
     bytePut64(&cursor, UV__DISK_FORMAT);
     bytePut64(&cursor, 0);
     bytePut64(&cursor, snapshot->configuration_index);
@@ -675,18 +751,14 @@ int UvSnapshotPut(struct raft_io *io,
     crc = byteCrc32(&put->meta.header[2], sizeof(uint64_t) * 2, 0);
     crc = byteCrc32(put->meta.bufs[1].base, put->meta.bufs[1].len, crc);
 
-    cursor = &put->meta.header[1];
+    cursor = (uint8_t *)&put->meta.header[1];
     bytePut64(&cursor, crc);
 
-    /* - If the trailing parameter is set to 0, it means that we're restoring a
-     * snapshot. Submit a barrier request setting the next append index to the
-     * snapshot's last index + 1.
-     * - When we are only writing a snapshot during normal operation, we close
-     * all current open segments. New writes can continue on newly opened
-     * segments that will only contain entries that are newer than the snapshot,
-     * and we don't change append_next_index. */
-    next_index = (trailing == 0) ? (snapshot->index + 1) : uv->append_next_index;
-    rv = UvBarrier(uv, next_index, &put->barrier, uvSnapshotPutBarrierCb);
+    /* Allocate two temporary files to hold the metadata and snapshot files. */
+    uv->snapshot_put_work.data = put;
+    rv = uv_queue_work(uv->loop, &uv->snapshot_put_work,
+                       uvSnapshotPutWorkAllocateCb,
+                       uvSnapshotPutAfterWorkAllocateCb);
     if (rv != 0) {
         goto err_after_configuration_encode;
     }
@@ -795,4 +867,28 @@ err:
     return rv;
 }
 
+static void uvSnapshotPutRetryCloseCb(uv_handle_t *handle)
+{
+    struct uv *uv = handle->data;
+    assert(uv->closing);
+    uv->snapshot_put_retry.data = NULL;
+    uvMaybeFireCloseCb(uv);
+}
+
+void UvSnapshotClose(struct uv *uv)
+{
+    if (uv->snapshot_put_retry.data != NULL) {
+        if (uv->snapshot_put_retry.data != uv) {
+            struct uvSnapshotPut *put = uv->snapshot_put_retry.data;
+            put->status = RAFT_CANCELED;
+            uvSnapshotPutFinish(put);
+            UvUnblock(uv);
+            uv->snapshot_put_retry.data = uv;
+        }
+        uv_timer_stop(&uv->snapshot_put_retry);
+        uv_close((uv_handle_t *)&uv->snapshot_put_retry,
+                 uvSnapshotPutRetryCloseCb);
+    }
+}
+
 #undef tracef
