@@ -13,7 +13,8 @@ struct UvTcp
     struct uv_loop_s *loop;              /* Event loop */
     raft_id id;                          /* ID of this raft server */
     const char *address;                 /* Address of this raft server */
-    struct uv_tcp_s listener;            /* Listening TCP socket handle */
+    unsigned n_listeners;                /* Number of listener sockets */
+    struct uv_tcp_s *listeners;          /* Listener sockets */
     raft_uv_accept_cb accept_cb;         /* Call after accepting a connection */
     queue accepting;                     /* Connections being accepted */
     queue connecting;                    /* Pending connection requests */
