@@ -1,4 +1,4 @@
-#include "../lib/cluster.h"
+#include "../lib/legacy.h"
 #include "../lib/runner.h"
 
 /******************************************************************************
@@ -25,7 +25,7 @@ static void *setup(const MunitParameter params[], MUNIT_UNUSED void *user_data)
     SETUP_CLUSTER(0);
     CLUSTER_BOOTSTRAP;
     CLUSTER_RANDOMIZE;
-    CLUSTER_START;
+    CLUSTER_START();
     CLUSTER_STEP_UNTIL_HAS_LEADER(10000);
     return f;
 }
@@ -163,7 +163,7 @@ TEST(replication, partitioned, setup, tear_down, 0, _params)
         raft_fixture_desaturate(&f->cluster, leader_id - 1, raft->id - 1);
     }
 
-    //TODO this fails with seed 0x3914306f
+    // TODO this fails with seed 0x3914306f
     CLUSTER_STEP_UNTIL_HAS_LEADER(30000);
 
     /* Re-try now to append the entry. */
