@@ -8,6 +8,8 @@
 #define DEFAULT_ELECTION_TIMEOUT 100
 #define DEFAULT_HEARTBEAT_TIMEOUT 50
 #define DEFAULT_NETWORK_LATENCY 10
+#define DEFAULT_SNAPSHOT_THRESHOLD 64
+#define DEFAULT_SNAPSHOT_TRAILING 32
 #define DEFAULT_DISK_LATENCY 10
 #define DEFAULT_DISK_SIZE 256 /* In bytes */
 
@@ -413,7 +415,9 @@ static void serverInit(struct test_server *s,
     s->cluster = cluster;
     s->network_latency = DEFAULT_NETWORK_LATENCY;
     s->disk_latency = DEFAULT_DISK_LATENCY;
-    s->snapshot_install = false;
+    s->snapshot.threshold = DEFAULT_SNAPSHOT_THRESHOLD;
+    s->snapshot.trailing = DEFAULT_SNAPSHOT_TRAILING;
+    s->snapshot.installing = false;
     s->running = false;
 }
 
@@ -454,6 +458,8 @@ static void dropReceiveEvent(struct step *step)
                 &event->receive.message->install_snapshot.conf);
             raft_free(event->receive.message->install_snapshot.data.base);
             break;
+        default:
+            break;
     }
 
     free(event->receive.message);
@@ -505,7 +511,7 @@ static void serverCancelPending(struct test_server *s)
             struct step *current;
             struct raft_message *message;
             current = QUEUE_DATA(head, struct step, queue);
-            munit_assert_int(current->event.type, ==, RAFT_RECEIVE);
+            munit_assert_uint(current->event.type, ==, RAFT_RECEIVE);
             message = current->event.receive.message;
             if (message->server_id /* sender */ == s->raft.id) {
                 step = current;
@@ -649,9 +655,9 @@ static void serverProcessSnapshot(struct test_server *s,
 {
     struct step *step = munit_malloc(sizeof *step);
 
-    munit_assert_false(s->snapshot_install);
+    munit_assert_false(s->snapshot.installing);
 
-    s->snapshot_install = true;
+    s->snapshot.installing = true;
 
     step->id = s->raft.id;
 
@@ -707,6 +713,8 @@ static void serverProcessMessages(struct test_server *s,
                 serverFillInstallSnapshot(
                     s, &event->receive.message->install_snapshot);
                 break;
+            default:
+                break;
         }
 
         QUEUE_PUSH(&s->cluster->send, &step->queue);
@@ -724,7 +732,7 @@ static void serverMaybeTakeSnapshot(struct test_server *s)
         last_snapshot_index = s->disk.snapshot->metadata.index;
     }
 
-    if (raft_commit_index(r) - last_snapshot_index < r->snapshot.threshold) {
+    if (raft_commit_index(r) - last_snapshot_index < s->snapshot.threshold) {
         return;
     }
 
@@ -746,7 +754,7 @@ static void serverMaybeTakeSnapshot(struct test_server *s)
     event->snapshot.metadata.configuration_index =
         r->configuration_committed_index;
 
-    event->snapshot.trailing = r->snapshot.trailing;
+    event->snapshot.trailing = s->snapshot.trailing;
 
     QUEUE_PUSH(&s->cluster->steps, &step->queue);
 }
@@ -979,7 +987,7 @@ static void serverCompleteEntries(struct test_server *s, struct step *step)
         diskAddEntry(&s->disk, &entries[i]);
     }
 
-    if (!s->snapshot_install) {
+    if (!s->snapshot.installing) {
         rv = serverStep(s, event);
         munit_assert_int(rv, ==, 0);
     }
@@ -996,7 +1004,7 @@ static void serverCompleteSnapshot(struct test_server *s, struct step *step)
     struct raft_event *event = &step->event;
     int rv;
 
-    s->snapshot_install = false;
+    s->snapshot.installing = false;
 
     snapshot->metadata.index = event->persisted_snapshot.metadata.index;
     snapshot->metadata.term = event->persisted_snapshot.metadata.term;
@@ -1239,6 +1247,22 @@ void test_cluster_set_network_latency(struct test_cluster *c,
     server->network_latency = latency;
 }
 
+void test_cluster_set_snapshot_threshold(struct test_cluster *c,
+                                         raft_id id,
+                                         unsigned threshold)
+{
+    struct test_server *server = clusterGetServer(c, id);
+    server->snapshot.threshold = threshold;
+}
+
+void test_cluster_set_snapshot_trailing(struct test_cluster *c,
+                                        raft_id id,
+                                        unsigned trailing)
+{
+    struct test_server *server = clusterGetServer(c, id);
+    server->snapshot.trailing = trailing;
+}
+
 void test_cluster_set_disk_latency(struct test_cluster *c,
                                    raft_id id,
                                    unsigned latency)
@@ -1361,7 +1385,7 @@ static void clusterEnqueueReceives(struct test_cluster *c)
         struct raft_message *message;
         head = QUEUE_HEAD(&c->send);
         step = QUEUE_DATA(head, struct step, queue);
-        munit_assert_int(step->event.type, ==, RAFT_RECEIVE);
+        munit_assert_uint(step->event.type, ==, RAFT_RECEIVE);
         QUEUE_REMOVE(&step->queue);
         message = step->event.receive.message;
 
