@@ -46,6 +46,18 @@ TEST(byteCrc32, invalid, NULL, NULL, 0, NULL)
 
 SUITE(byteFlip)
 
+/* Convert a 16-bit number. */
+TEST(byteFlip, 16, NULL, NULL, 0, NULL)
+{
+    uint16_t value;
+    unsigned i;
+    value = byteFlip16(0x0100);
+    for (i = 0; i < 2; i++) {
+        munit_assert_int(*((uint8_t *)&value + i), ==, i);
+    }
+    return MUNIT_OK;
+}
+
 /* Convert a 32-bit number. */
 TEST(byteFlip, 32, NULL, NULL, 0, NULL)
 {
@@ -81,7 +93,7 @@ SUITE(byteGetString)
 TEST(byteGetString, success, NULL, NULL, 0, NULL)
 {
     uint8_t buf[] = {'h', 'e', 'l', 'l', 'o', 0};
-    const void *cursor = buf;
+    const uint8_t *cursor = buf;
     munit_assert_string_equal(byteGetString(&cursor, sizeof buf), "hello");
     munit_assert_ptr_equal(cursor, buf + sizeof buf);
     return MUNIT_OK;
@@ -90,7 +102,7 @@ TEST(byteGetString, success, NULL, NULL, 0, NULL)
 TEST(byteGetString, malformed, NULL, NULL, 0, NULL)
 {
     uint8_t buf[] = {'h', 'e', 'l', 'l', 'o', 'w'};
-    const void *cursor = buf;
+    const uint8_t *cursor = buf;
     munit_assert_ptr_equal(byteGetString(&cursor, sizeof buf), NULL);
     munit_assert_ptr_equal(cursor, buf);
     return MUNIT_OK;
@@ -98,19 +110,19 @@ TEST(byteGetString, malformed, NULL, NULL, 0, NULL)
 
 /******************************************************************************
  *
- * byteGet64Unaligned
+ * byteGet64
  *
  *****************************************************************************/
 
-SUITE(byteGet64Unaligned)
+SUITE(byteGet64)
 
-TEST(byteGet64Unaligned, success, NULL, NULL, 0, NULL)
+TEST(byteGet64, success, NULL, NULL, 0, NULL)
 {
     uint8_t *buf = munit_malloc(sizeof(uint64_t) * 2);
-    void *cursor1 = buf + 1;
-    const void *cursor2 = buf + 1;
-    bytePut64Unaligned(&cursor1, 1);
-    munit_assert_int(byteGet64Unaligned(&cursor2), ==, 1);
+    uint8_t *cursor1 = buf + 1;
+    const uint8_t *cursor2 = buf + 1;
+    bytePut64(&cursor1, 1);
+    munit_assert_ullong(byteGet64(&cursor2), ==, 1);
     free(buf);
     return MUNIT_OK;
 }
